#include "../Iinstr.hpp"


class Csth : public Iinstr {
	public:
		Csth();
		void evalInstr();
		void execInstr();
		std::string toString();
		Iinstr* getNew();
};

/**
 * Name:      create_instruction
 * Purpose:   if compiled as shared library, this functions creates the 
              instruction object

 * Returns:   pointer to instruction object
 */
extern "C" Iinstr* create_instruction() {
    return new Csth();
}

Iinstr* Csth::getNew()
{
    return new Csth();
}

/**
 * Name:      destroy_instruction
 * Purpose:   if compiled as shared library, this functions destoys the 
              instruction object

 * Parameter: IInstruction - the instruction object to delete
 */
extern "C" void destroy_instruction(Iinstr* p) {
    delete p;
}

Csth::Csth()
{
	opcode = B5(10001);
	name = "sth";
}

void Csth::evalInstr()
{
	this->m_s = true;

	dynamic_bitset<> immb = argbits;
	immb.resize(15);
	this->m_imm = this->generate15ImmSign(immb.to_ulong());

	argbits >>= 15;
	m_ra = this->getRegister(argbits);
	argbits >>= 4;
	m_rd = this->getRegister(argbits);

}

#include <iomanip>
#define AS_HEX std::hex << setw(8) << setfill('0')

void Csth::execInstr()
{
	//cout << "should exec " << this->toString() << endl;
	CDat addr = this->m_cpu->getRegister(this->m_ra);
	addr += m_imm;
	MEMORY_ADDRESS temp = (addr & (~(BYTE_COUNT-1)));
	CDat offset = addr-temp;
	if(offset%2 == 1) {
		cerr << "you try to get an invalid halfword!" << endl;
		return;
	}
	CDat val = this->m_cpu->getRAM(temp);

	val = val & ~( ((1<<16)-1) << (offset*8));

	//data to insert
	CDat source = this->m_cpu->getRegister(this->m_rd);
	source = (source & (((1<<16)-1))) << (offset*8);

	val += source;

	this->m_cpu->setRAM(temp, val);
}

std::string Csth::toString()
{
	stringstream op;
	op << this->getName();

	op << this->getConditionFlag() << " r" << m_rd << ", " << m_imm << "(r" <<  m_ra << ")";

	return op.str();
}
